@@ -8,6 +8,7 @@ import uuid
 from datetime import datetime
 from pathlib import Path
 from secrets import token_hex
+from time import sleep, time
 from typing import Any, Dict, Optional
 
 import pkg_resources
@@ -84,6 +85,7 @@ class SmartLab:
         jobs: Optional[int] = None,
         skip_exists: bool = False,
         force: bool = True,
+        timeout: int = 3600,
     ) -> Path:
         """Automatically downloads files from DVC storage of given remote.
 
@@ -94,6 +96,7 @@ class SmartLab:
             jobs: Parallelism level for DVC to download data from remote storage.
             skip_exists: Skips fetching DVC cache if target path exists.
             force: Overrides workspace files without asking.
+            timeout: Waiting time for release of DVC lock.
 
         Returns:
             Path to downloaded file or directory.
@@ -110,7 +113,12 @@ class SmartLab:
             if target.is_mount():
                 raise RuntimeError(f"Target {target_path} is not versioned by DVC.")
 
-        DVCRepo(root_dir).pull(
+        dvc_repo = DVCRepo(root_dir)
+        self._wait_on_dvc_lock_release(
+            dvc_repo=dvc_repo,
+            timeout=timeout,
+        )
+        dvc_repo.pull(
             targets=str(target),
             remote=remote or self.remote,
             force=force,
@@ -130,6 +138,7 @@ class SmartLab:
         remote: Optional[str] = None,
         sync: bool = True,
         jobs: Optional[int] = None,
+        timeout: int = 3600,
     ) -> None:
         """Uploads files to DVC storage of given remote.
 
@@ -139,6 +148,7 @@ class SmartLab:
             remote: Name of the remote storage to be uploaded.
             sync: Uploads to remote storage, otherwise save only to local cache.
             jobs: Parallelism level for DVC to upload data to remote storage.
+            timeout: Waiting time for release of DVC lock.
 
         """
         root_dir = DVCRepo.find_root(root=target if target.is_dir() else target.parent)
@@ -151,7 +161,12 @@ class SmartLab:
                 to_remote=True,
             )
 
-        DVCRepo(root_dir).add(
+        dvc_repo = DVCRepo(root_dir)
+        self._wait_on_dvc_lock_release(
+            dvc_repo=dvc_repo,
+            timeout=timeout,
+        )
+        dvc_repo.add(
             targets=str(target),
             recursive=recursive,
             **args,
@@ -217,3 +232,19 @@ class SmartLab:
                     version_number = max(version_number, int(number))
 
         return version_number + 1
+
+    @staticmethod
+    def _wait_on_dvc_lock_release(
+        dvc_repo: DVCRepo,
+        timeout: int = 3600,
+        delay: float = 2.0,
+    ) -> None:
+        time_start = time()
+        while dvc_repo.lock.is_locked:
+            print("Waiting for release of DVC lock")
+            sleep(delay)
+            if time() - time_start > timeout:
+                raise TimeoutError(
+                    "DVC repository is locked. Most likely another DVC process is running "
+                    "or was terminated abruptly."
+                )
